@@ -45,7 +45,7 @@ class NotificationFetcher(dbus.service.Object):
         self.context = []
 
         if len(self.dm.tree):
-            self._id = self.dm.tree.last.id + 1
+            self._id = self.dm.tree.best.id + 1
 
         name = dbus.service.BusName(DBUS_PATH, dbus.SessionBus())
         super().__init__(name, "/org/freedesktop/Notifications")
@@ -83,7 +83,7 @@ class NotificationFetcher(dbus.service.Object):
 
     @dbus.service.method(DBUS_PATH, in_signature="", out_signature="ssss")
     def GetServerInformation(self):
-        return "i3notifier", "github.com/sencer/i3-notifier", "0.12", "1.2"
+        return "i3notifier", "github.com/sencer/i3-notifier", "0.16", "1.2"
 
     @dbus.service.method(DBUS_PATH, in_signature="susssasa{ss}i", out_signature="u")
     def Notify(
@@ -156,7 +156,7 @@ class NotificationFetcher(dbus.service.Object):
             )
             notification.timer.start()
 
-        self._notifications_updated()
+        self._notifications_updated(0)  # 0: notification added
         return id
 
     @dbus.service.method(DBUS_PATH, in_signature="", out_signature="uu")
@@ -170,7 +170,7 @@ class NotificationFetcher(dbus.service.Object):
 
     @dbus.service.method(DBUS_PATH, in_signature="", out_signature="")
     def SignalNotificationCount(self):
-        self._notifications_updated()
+        self._notifications_updated(2)  # 2: manual
 
     # Signals
 
@@ -182,14 +182,19 @@ class NotificationFetcher(dbus.service.Object):
     def NotificationClosed(self, id, reason):
         logger.info(f"NotificationClosed signalled for {id} due to {reason}.")
 
-    @dbus.service.signal(DBUS_PATH, signature="uu")
-    def NotificationsUpdated(self, num, urgency):
+    @dbus.service.signal(DBUS_PATH, signature="uuus")
+    def NotificationsUpdated(self, mode, num, urgency, single_line):
         logger.info(f"Notifications updated.")
 
     # Internal methods
 
-    def _notifications_updated(self):
-        self.NotificationsUpdated(len(self.dm.tree), self.dm.tree.urgency or 0)
+    def _notifications_updated(self, mode):
+        self.NotificationsUpdated(
+            mode,
+            len(self.dm.tree),
+            (self.dm.tree.urgency or 0),
+            self.dm.last.single_line() if self.dm.last else "",
+        )
 
     def _process_hooks(self, notification, hook_name):
         logger.info(f"Processing {hook_name}")
@@ -207,18 +212,14 @@ class NotificationFetcher(dbus.service.Object):
     def _remove_notification(self, key, reason):
         logger.info(f"Attempting to remove notification {key} since {reason}")
         self.dm.remove_notification(key, self.context)
-        self._notifications_updated()
+        self._notifications_updated(1)  # 1: deleted
         return self._update_context()
 
     def _show_notifications(self, row=0):
         notifications = self.dm.get_context(self.context).notifications
 
         items = sorted(
-            [
-                (k, v) if len(v) > 1 else (v.last.id, v.last)
-                for k, v in notifications.items()
-            ],
-            key=lambda x: (-x[1].urgency, -x[1].last.created_at),
+            notifications.items(), key=lambda x: (-x[1].urgency, -x[1].best.created_at)
         )
 
         selected, op = self.gui.show_notifications([item[1] for item in items], row)
@@ -240,20 +241,20 @@ class NotificationFetcher(dbus.service.Object):
         key, notification = items[selected]
 
         if op == Operation.SELECT:
-            if isinstance(notification, Notification):
+            if len(notification) == 1:
                 logger.info("Selection is a singleton. Invoking default action.")
-                self.context = self.dm.map[notification.id]
+                self.context = self.dm.map[notification.best.id]
 
-                if self._process_hooks(notification, "pre_action_hooks"):
-                    self.ActionInvoked(notification.id, "default")
+                if self._process_hooks(notification.best, "pre_action_hooks"):
+                    self.ActionInvoked(notification.best.id, "default")
                 else:
                     logger.info(f"Skipping action for {notification.id}.")
 
-                if self._process_hooks(notification, "post_action_hooks"):
+                if self._process_hooks(notification.best, "post_action_hooks"):
                     self._remove_notification(
-                        notification.id, RemoveReason.ACTION_INVOKED
+                        notification.best.id, RemoveReason.ACTION_INVOKED
                     )
-                    self.NotificationClosed(notification.id, 2)
+                    self.NotificationClosed(notification.best.id, 2)
                 else:
                     logger.info(
                         f"Skipping CloseNotification (after action) for {notification.id}."
@@ -264,7 +265,8 @@ class NotificationFetcher(dbus.service.Object):
                 self._show_notifications()
         elif op == Operation.DELETE:
             context_changed = self._remove_notification(key, RemoveReason.USER_DELETED)
-            row = 0 if context_changed or len(notifications) == 1 else selected
+
+            row = 0 if context_changed else selected - (selected == len(notifications))
 
             if len(self.dm.tree):
                 self._show_notifications(row)
