@@ -1,3 +1,4 @@
+import os.path
 import subprocess
 from enum import Enum
 
@@ -13,54 +14,44 @@ class RofiGUI:
     _separator = b"\x01"
     _args = [
         "-dmenu",
-        "-p",
-        "Notifications",
-        "-eh",
-        "2",
-        "-width",
-        "-40",
         "-markup-rows",
         "-i",
         "-format",
         "i",
         "-sep",
         r"\x01",
-        "--kb-accept-custom",
-        "",
-        "--kb-accept-alt",
-        "",
-        "-kb-custom-1",
-        "Ctrl+Delete",
-        "-show-icons",
-        "-columns",
-        "1",
-        "-lines",
-        "10",
     ]
 
-    __slots__ = 'cmd'
+    __slots__ = "cmd"
 
-    def __init__(self, *args, cmd=None):
+    def __init__(self, *args, theme=None, cmd=None):
         self.cmd = cmd or "rofi"
         self._args.extend(args)
+        if theme is not None:
+            self._args.extend(
+                ["-theme", f"{os.path.dirname(__file__)}/rofi-theme/{theme}"]
+            )
 
     def show_notifications(self, notifications, row=0):
 
-        formatted_notifications = [
-            notification.formatted() for notification in notifications
-        ]
+        formatted_notifications = []
+        urgent = []
+        active = []
+        for i, notification in enumerate(notifications):
+            formatted_notifications.append(notification.formatted())
 
-        urgent = [
-            str(i)
-            for i, notification in enumerate(notifications)
-            if notification.urgency == 2
-        ]
+            if notification.urgency == 2:
+                urgent.append(str(i))
+
+            if len(notification) > 1:
+                active.append(str(i))
 
         proc = subprocess.Popen(
             [self.cmd]
             + self._args
             + ["-selected-row", str(row)]
-            + (["-u", ",".join(urgent)] if urgent else []),
+            + (["-u", ",".join(urgent)] if urgent else [])
+            + (["-a", ",".join(active)] if active else []),
             stdin=subprocess.PIPE,
             stdout=subprocess.PIPE,
         )
